/* implementing a stack using array */

#include <stdio.h>

#define STACK_MAX 8

#define STACK_ELT_FMT "d"
typedef int stack_elt;

struct stack {
    size_t length;
    stack_elt values[STACK_MAX];
};

enum stack_error {
    STACK_ERR_OK,
    STACK_ERR_UNDERFLOW,
    STACK_ERR_OVERFLOW,
    STACK_ERR_NOELEM
};

struct stack *
stack_new(void)
{
    struct stack *st = calloc(1, sizeof(*st));
    st->length = 0;
    return st;
}

enum stack_error
stack_push(struct stack *st, struct_elt elt)
{
    if (st->length >= STACK_MAX)
        return STACK_ERR_OVERFLOW;
    st->values[st->length++] = elt;
    return STACK_ERR_OK;
}

enum stack_error
stack_pop(struct stack *st, struct_elt *elt)
{
    if (st->length == 0)
        return STACK_ERR_UNDERFLOW;
    *elt = st->values[--st->length];
    return STACK_ERR_OK;
}

enum stack_error
stack_peek(struct stack *st, stack_elt *elt)
{
    if (st->length == 0)
        return STACK_ERR_NOELEM;
    *elt = st->values[st->length-1];
    return STACK_ERR_OK;
}

void
stack_print(struct stack *st)
{
    size_t i;
    for (i = 0; i < st->length; i++) {
        printf("\t%" STACK_ELT_FMT, st->values[i]);
    }
    puts("");
}

void
stack_print_error(enum stack_error err)
{
    switch (err) {
    case STACK_ERR_OK:
        fprintf(stderr, "ok\n");
        break;

    case STACK_ERR_OVERFLOW:
        fprintf(stderr, "stack overflow!\n");
        break;

    case STACK_ERR_UNDERFLOW:
        fprintf(stderr, "stack underflow!\n");
        break;

    case STACK_ERR_NOELEM:
        fprintf(stderr, "stack empty!\n");
        break;
    }
}

#define TRY(f) \
    err = f; \
    stack_print_error(err); \
    stack_print(t);


void
stack__test(void)
{
    struct stack *t = stack_new();
}

int
main(void)
{
    stack__test();
    return 0;
}



